; set filter parameters
; written for DATAdesign engine v2.+
; (c)1992 by Joachim Van der Auwera for PROGS
        section text
        include win1_dd_src_engine_aid_mac
        include win1_dd_src_engine_struct_h
        include win1_inc_keys_err
        include win1_inc_keys_k
*
; set filter parameters
; can handle up to 10 filter levels
; if compare = -1 (optional), than only filtered until previous level
; you can test whether
; - a field is or is not present
; - whether a certain char (any of eight per level) is present in the field (place=0) (zero bytes are not evaluated)
; - or as a certain char in the field (level>0) (zero bytes are not evaluated)
; - whether the mark status <,>,<=,>=,<> the given value (byte, left justified) (unsigned compare)
; - whether a value in a numerical field <,>,<=,>=,<> the given value (left justified)
;   (place=0 OR element doesn't exist -> levelstatus=FALSE)
; if indexid given, than filter parameters overwritten
; else new index created with filter params set, and index set as default index
; the indextree in not being built by this routine, needs call to IUPD
; parameters
p.id    equ     $4      ; buffer id or indexid (long word, optional)
p.c1    equ     $8      ; compare level 1 (optional, nnul)
p.f1    equ     $c      ; fieldid
p.p1    equ     $10     ; place in field
p.v1    equ     $14     ; 8.b value to compare, LEFT JUSTIFIED (char passed as word)!!
;...
p.c0    equ     $bc     ; compare level 10 (optional, nnul)
p.f0    equ     $c0     ; fieldid
p.p0    equ     $c4     ; place in field
p.v0    equ     $c8     ; 8.b value to compare, LEFT JUSTIFIED (char passed as word)!!
p.ret   equ     $d4     ; return long indexid
; errors
; itnf  invalid buffer or indexid
; imem  out of memory
cod_FILT xsub
        regsave d1-d7/a0-a2/a4/a5
        move.l  a1,a5           ; keep params in a safe place

        move.l  p.id(a5),d0     ; buffer id if passed
        tst.w   p.id-4(a5)      ; is the buffer id given ?
        bne.s   tbuff           ; yes -> ..
        xjsr    getbuff         ; get default buffer
tbuff   move.l  d0,a4           ; keep it
        xjsr    testbid         ; and test if it is valid
        beq.s   start           ; ok -> new index
        move.l  a4,d0
        xjsr    testiid         ; is it an indexid ?
        bne.s   codend          ; no -> forget it
        move.l  a4,a2           ; indexid in a2
        bra.s   setparms

start   ; first allocate the new index
        move.l  #i.len,d1       ; amount of mem needed
        xjsr    getmem          ; alocate it
        bne.s   codend          ; oops, out of memory
        linkin  b_index(a4),a0  ; link in this index
        move.l  a0,b_dindex(a4) ; and make it new default
        move.l  #'DATA',4(a0)   ; fill in flag
        move.l  #'desi',8(a0)
        move.l  #'gn i',12(a0)
        move.l  #'ndex',16(a0)
        move.l  a4,i_buffer(a0) ; bufferid for this index
        move.w  #-1,i_sort+i_cmp(a0)
        move.l  a0,a2           ; indexid in a2
setparms
        move.l  p.ret(a5),a1
        move.l  a2,(a1)         ; specify return value

        ; now fill in the compare data
        lea     i_filt(a2),a1   ; place to fill data in
        lea     p.c1(a5),a0     ; place to get it from
        moveq   #9,d1           ; copy 10 filter levels
cplp    move.w  2(a0),(a1)+     ; compare method
        move.w  6(a0),(a1)+     ; fieldid
        move.w  10(a0),(a1)+    ; place in field
        move.l  12(a0),(a1)+    ; value to compare
        move.l  16(a0),(a1)+
        adda.w  #20,a0          ; go to next param
        dbf     d1,cplp
codend
        subend

; routine to determine whether a record is visible or not
; needs a2 indexid
; boolean value returned in d0
recfilt xsub
        regsave d1-d7/a0-a4
; we need the bufferid
        move.l  i_buffer(a2),a4
; status = TRUE
        moveq   #-1,d6

        moveq   #9,d7           ; counter for filter levels
        lea     i_filt(a2),a2   ; we only need the filterdata
        move.w  i_fcmp(a2),d1   ; first level always has to be AND-ed
        bset    #c..and,d1
        move.w  d1,i_fcmp(a2)
; WHILE there is another filter level AND compare<>-1
while   cmpi.w  #-1,i_fcmp(a2)
        beq     return
;       levelstatus = FALSE
        moveq   #0,d5           ; init levelstatus to FALSE
;       find field specified
        move.b  i_ffld(a2),d2
        move.l  b_rlen(a4),d1
        move.l  b_mem(a4),a1
        xjsr    findfld
;       do we need to know if this field is empty -> set status (test for NOT)
        move.w  i_fcmp(a2),d1
        btst    #c..empty,d1
        beq.s   noempty
        asr.w   #8,d0
        move.w  d0,d5           ; levelststatus
        ext.l   d5
        bra     not             ; test for not
noempty
;       oops (cfr. find field) -> not found (levelstatus=FALSE)
        tst.l   d0
        bne     nxtwh
;       no problem -> text or raw THEN
noops   move.b  i_ffti(a2),d2   ; fieldtype
        cmpi.b  #ft.char,d2     ; text or raw
        bgt     numeric         ; no -> numerical field
;               as string ?? no -> skip
        move.w  i_fcmp(a2),d2
        btst    #c..str,d2      ; check
        beq.s   nostr
;               try to find given (max 8 letter) string
        moveq   #0,d2
        move.w  i_fplc(a2),d2   ; nr of char we need
        beq.s   strst
        subq.w  #1,d2           ; subtract one !! first char is at pos zero
strst   move.l  rf_len(a1),d4   ; length of field
        sub.l   d2,d4           ; number of chars to test
        lea     rf_fld(a1,d2.w),a1; to start, and skip field header

strlp   subq.l  #1,d4
        bmi     not
        moveq   #0,d2           ; counter for character to compare
strchar
        move.b  i_fval(a2,d2.w),d1      ; is there another char
        beq.s   strgot          ; no -> found it

        cmp.l   d2,d4           ; can we compare another char
        blt.s   strlpn          ; no -> try next position
        move.b  (a1,d2.w),d3    ; char to test
;
        move.w  i_fcmp(a2),d0
        btst    #c..case,d0
        bne.s   nocase4
        cmpi.b  #'a',d3
        blt.s   ecase4          ; not lowercase -> ..
        cmpi.b  #'z',d3
        bgt.s   ecase4          ; not lowercase -> ..
        add.b   #'A'-'a',d3     ; convert to uppercase
ecase4
        cmpi.b  #'a',d1
        blt.s   ecase5          ; not lowercase -> ..
        cmpi.b  #'z',d1
        bgt.s   ecase5          ; not lowercase -> ..
        add.b   #'A'-'a',d1     ; convert to uppercase
ecase5
nocase4
;
        cmp.b   d1,d3   ; compare
        bne.s   strlpn
        addq.w  #1,d2   ; next char
        cmpi.w  #8,d2   ; compare
        blt     strchar ; there is another one -> compare next char
strgot
        moveq   #-1,d5  ; found it (set levelstatus)
        bra     not     ; set new filterstatus
;                       IF try at next position -> do so
strlpn
        addq.l  #1,a1           ; take next position
        tst.w   i_fplc(a2)      ; do we have to compare it
        beq     strlp           ; yes -> do so
        bra     not             ; no -> set result

;               place<>0) -> get that char (if exists)
nostr   tst.w   i_fplc(a2)      ; test one char only
        beq.s   nofrst
        moveq   #0,d2
        move.w  i_fplc(a2),d2   ; nr of char we need
        cmp.l   rf_len(a1),d2   ; does this char exist
        bgt     not
        move.b  rf_fld-1(a1,d2.w),d3
;                       c..case ?? -> convert to upper case
        move.w  i_fcmp(a2),d0
        btst    #c..case,d0
        bne.s   ecase
        cmpi.b  #'a',d3
        blt.s   ecase           ; not lowercase -> ..
        cmpi.b  #'z',d3
        bgt.s   ecase           ; not lowercase -> ..
        add.b   #'A'-'a',d3     ; convert to uppercase
ecase
;                       FOR 8 chars
        moveq   #7,d2           ; loop counter
frstclp
;                               get char (=0 -> NEXT)
        move.b  i_fval(a2,d2.w),d1      ; char to compare
        beq.s   frstdif
;                               c..case ?? -> convert to upper case
        move.w  i_fcmp(a2),d0
        btst    #c..case,d0
        bne.s   ecase1
        cmpi.b  #'a',d1
        blt.s   ecase1          ; not lowercase -> ..
        cmpi.b  #'z',d1
        bgt.s   ecase1          ; not lowercase -> ..
        add.b   #'A'-'a',d1     ; convert to uppercase
ecase1
;                               compare -> equal -> levelstatus = TRUE
        cmp.b   d1,d3   ; compare
        bne.s   frstdif
        moveq   #-1,d5
        bra.s   not
frstdif dbf     d2,frstclp
        bra.s   not
;               else ->
nofrst
;                      LOOP    take next char
        move.l  rf_len(a1),d4   ; length of field
        addq.l  #rf_fld,a1      ; skip field header
inlp    subq.l  #1,d4
        bmi.s   not
        move.b  (a1)+,d3        ; char to test
;                               c..case ?? -> convert to upper case
        move.w  i_fcmp(a2),d0
        btst    #c..case,d0
        bne.s   ecase2
        cmpi.b  #'a',d3
        blt.s   ecase2          ; not lowercase -> ..
        cmpi.b  #'z',d3
        bgt.s   ecase2          ; not lowercase -> ..
        add.b   #'A'-'a',d3     ; convert to uppercase
ecase2
;                               FOR 8 chars
        moveq   #7,d2           ; loop counter
inclp
;                                       get char (=0 -> NEXT)
        move.b  i_fval(a2,d2.w),d1      ; char to compare
        beq.s   indif
;                                       c..case ?? -> convert to upper case
        move.w  i_fcmp(a2),d0
        btst    #c..case,d0
        bne.s   ecase3
        cmpi.b  #'a',d1
        blt.s   ecase3          ; not lowercase -> ..
        cmpi.b  #'z',d1
        bgt.s   ecase3          ; not lowercase -> ..
        add.b   #'A'-'a',d1     ; convert to uppercase
ecase3
;                                       compare -> equal -> levelstatus = TRUE
        cmp.b   d1,d3   ; compare
        bne.s   indif
        moveq   #-1,d5
        bra.s   not
indif   dbf     d2,inclp
;                       UNTIL levelstatus OR no next char
        tst.l   d5
        beq     inlp
;               c..not ?? -> NOT levelstatus
not     move.w  i_fcmp(a2),d0
        btst    #c..not,d0
        beq     nxtwh
        eori.l  #-1,d5
        bra     nxtwh
;       else (numerical fields or date or mark) (date not implemented yet)
numeric
;               take value to compare (in field or..) (type specific)
;               convert both values to suitable format to compare
        move.w  i_fcmp(a2),d0
        btst    #c..mark,d0     ; mark status ??
        beq.s   value           ; no -> just compare a value
        moveq   #0,d0
        moveq   #0,d1
        move.b  i_fval(a2),d0
        move.b  b_mark(a4),d1
        cmp.w   d0,d1           ; compare
        bra     lstat           ; handle according to compare
value   ; we have to get a value to compare
        cmpi.b  #ft.long,i_ffti(a2)
        bmi     fshort
        beq.s   flong
fdouble ; fill in a double to compare
        moveq   #0,d0
        move.w  i_plc(a2),d0
        beq     nxtwh
        add.l   d0,d0
        add.l   d0,d0
        add.l   d0,d0
        cmp.l   rf_len(a1),d0   ; does this item exist ?
        bgt     nxtwh           ; no -> ..
        moveq   #0,d1           ; word to eor
        moveq   #0,d2           ; word to process (field)
        moveq   #0,d3           ; word to eor
        moveq   #0,d4           ; word to process (value)
        lea     rf_fld-8(a1,d0.l),a3
        move.w  (a3)+,d2        ; word with sign bit
        lea     i_fval(a2),a0
        move.w  (a0)+,d4
        bset    #15,d2          ; sign ??
        beq.s   fdf1
        bclr    #15,d2
        moveq   #-1,d1
fdf1    bset    #15,d4          ; sign ??
        beq.s   fdf2
        bclr    #15,d2
        moveq   #-1,d3
fdf2
        cmp.l   d4,d2           ; compare word with sign
        bne     lstat
        moveq   #2,d0
dbllp   move.w  (a0)+,d4
        eor.w   d3,d4
        move.w  (a3)+,d2
        eor.w   d1,d2
        cmp.l   d4,d2
        bne.s   lstat
        dbf     d0,dbllp
        moveq   #0,d0
        bra.s   lstat

flong   ; fill in a long to sort
        moveq   #0,d3
        move.w  i_plc(a2),d3
        beq     nxtwh
        add.l   d3,d3
        add.l   d3,d3
        cmp.l   rf_len(a1),d3   ; does this item exist ?
        bgt.s   nxtwh           ; no -> ..
        move.l  rf_fld-4(a1,d3.l),d1
        subi.l  #$80000000,d1
        move.l  i_fval(a2),d3
        subi.l  #$80000000,d3
        moveq   #0,d0
        moveq   #0,d2
        swap    d1
        swap    d3
        move.w  d1,d0
        move.w  d3,d2
        cmp.l   d2,d0
        bne.s   lstat
        swap    d1
        swap    d3
        move.w  d1,d0
        move.w  d3,d2
        cmp.l   d2,d0
        bra.s   lstat

fshort  ; fill in a short to sort
        moveq   #0,d3
        move.w  i_plc(a2),d3
        beq.s   nxtwh
        add.l   d3,d3
        cmp.l   rf_len(a1),d3   ; does this item exist ?
        bgt.s   nxtwh           ; no -> ..
        moveq   #0,d1
        move.w  rf_fld-2(a1,d3.l),d1
        subi.w  #$8000,d1
        moveq   #0,d2
        move.w  i_fval(a2),d2
        subi.w  #$8000,d1
        cmp.l   d2,d1

;               set levelstatus according to compare
lstat   bgt.s   larg
        blt.s   smal
        move.w  i_fcmp(a2),d0
        btst    #c..equ,d0
        beq.s   nxtwh
        moveq   #-1,d5
        bra.s   nxtwh
smal    move.w  i_fcmp(a2),d0
        btst    #c..smal,d0
        beq.s   nxtwh
        moveq   #-1,d5
        bra.s   nxtwh
larg    move.w  i_fcmp(a2),d0
        btst    #c..larg,d0
        beq.s   nxtwh
        moveq   #-1,d5
;       if c..and -> AND levelstatus to level
nxtwh   move.w  i_fcmp(a2),d0
        btst    #c..and,d0
        beq.s   noand
        and.l   d5,d6
        bra.s   nxt
;            else ->  OR levelstatus to level
noand   or.l    d5,d6
; end while
nxt     adda.w  #i.flen,a2
        dbf     d7,while
; return level
return  move.l  d6,d0
        subend
;
        end
