/* special program to convert an old DATAdesign file
 * to a file which is compatible with DATAdesign engine v3.+
 *
 * we explicitly need the DATAdesign engine for this
 * (c) 16 January 1993 by Joachim Van der Auwera for
 *				  PROGS, PROfessional & Graphical Software
 */
#include "DATAdesign.h"
#include "qdos.h"
#include "stdio.h"
#include "string.h"

char _PROG_NAME[] = "OLDtoNEW, DATAdesign";
char *_endmsg=NULL;
int err;
#define eof (err==ERR_EF)

char filenam[64],destnam[64];	/* filenames to convert */
char temp[8192];		/* temporary space for record */
chanid_t in;			/* file for input */
int r;					/* to keep a result */
short fldlen, nrfld, reclen;
long offset;

/* routine to read a record */
void readrec()
		{
		/* read length of record */
		err=io_fstrg(in,-1,(char *)&reclen,(short)2);
		/* read data in record */
		if (!eof) err=io_fstrg(in,-1,temp,reclen);
		}

/* routine to set a record in the file */
void setrec()
		{
		char *start=temp,*pos=temp;
		short field=1; int len=0;

		DDnewrec(0);	  /* it has to be a new record */

		/* now run through the record */
		/* we have to replace all CR by NL */
		/* when we encounter a NL, the field is done */
		for (;reclen;reclen--) {
				if (*pos=='\n') {        /* end of field reached */
						if (len)		/* if there was a field */
								DDsetfield(0,field,len,start);
						len=-1; 		/* it will immediatly be incremented! */
						field++;
						start=pos+1;
						}
				if (*pos=='\r') *pos='\n';
				pos++; len++;
				}
		if (len) DDsetfield(0,field,len,start);    /* set remaining field */

		DDimplement(0);   /* and implement that record */
		}

/* routine to set the fields */
void setfields()
		{
		char *name=&temp[4];
		char *pos=temp;
		int i,j;

		io_fstrg(in,-1,temp,fldlen);

		nrfld= *( (short *)pos );		/* no of fields */
		fldlen= *( (short *)(pos+2) );	/* length of fieldname */
		for (i=nrfld+1;i;i--) {
				/* set end of string marker */
				*(name+fldlen-1)='\0';
				/* remove trailing spaces */
				for (j=fldlen-2;*(name+j)==' ';j--) *(name+j)='\0';
				DDaddfield(0,name,(short)2);  /* add char field */
				name+=fldlen;
				}
		}

/* actual routine which does the conversion */
void convert()
		{
		int i;

		/* open the input file */
		in=io_open(filenam, OLD_SHARE);
		if ( in<0 ) {
				puts ("ERROR while opening file");
				return;
				}
		/* see if the file has the correct type */
		io_fstrg(in,-1,temp,(short)10);
		temp[10]='\0';
		if (!strcmp("DATAdesign",temp)) offset=12+8;
		else if (!strcmp("DATAdsgn01",temp)) offset=12+40;
		else if (!strcmp("DATAdsgn02",temp)) offset=12+40+28;
		else {  puts ("ERROR : not a DATAdesign file");
				io_close(in);
				return;
				}
		io_fstrg(in,-1,(char *)&fldlen,(short)2);	  /* length of fielddata */
		if (fldlen==0x2066) {     /* ==" f", so as header="DATAdesign file " */
				puts ("this file doesn't need conversion");
				io_close(in);
				return;
				}
		offset+=fldlen; 						/* start of records */
		/* now create the DATAdesign file */
		r=DDnewfile(destnam);
		if (r) { puts ("ERROR on creating new file");
				io_close(in);
				return;
				}
		setfields();			/* set all existing fields,
								   they will have fieldid's 1..nr of fields */
		fs_pos(in,offset,0);	/* set correct file position (absolute) */

		/* copy all the records */
		readrec();
		while (!eof) {
				setrec();
				readrec();
				}

		/* close the input file */
		io_close (in);

		/* and save and close the output file */
		r=DDsavefile(0,NULL,NULL,(short)1);  /* save default with default name */
		if (r)	puts ("ERROR while saving file");
		DDunusebuffer(0);		  /* stop using default file */
		}

void main()
		{
		if (DDinit()) {    /* initialise DATAdesign engine */
				puts ("DATAdesign engine not found, press a key");
				getc(stdin);
				return;
				}

		puts ("This is the DATAdesign file converter");
		puts ("  To convert old DATAdesign files to DATAdesign files");
		puts ("  which are suitable for DATAdesign v3 and higher");
		while (1)
				{
				puts ("Source filename (enter to end) :");
				gets (filenam);
				if ( !strlen(filenam) ) return;
				puts ("Destination filename (will be overwritten) :");
				gets (destnam);
				convert();
				puts ("Conversion finished ..\n");
				}

		DDfree();  /* release the DATAdesign engine */
		}
